import { __decorate, __param } from 'tslib';
import { ɵɵdefineInjectable, ɵɵinject, Injectable, Component, EventEmitter, Input, Output, ChangeDetectorRef, Inject, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { Validators, FormBuilder, FormsModule, ReactiveFormsModule } from '@angular/forms';
import { MatSnackBar, MatSnackBarModule } from '@angular/material/snack-bar';
import { MatDialogRef, MatDialog, MAT_DIALOG_DATA, MatDialogModule } from '@angular/material/dialog';
import { moveItemInArray, DragDropModule } from '@angular/cdk/drag-drop';
import { valid } from 'semver';
import * as sha1_ from 'js-sha1';
import { MatIconModule } from '@angular/material/icon';
import { MatTooltipModule } from '@angular/material/tooltip';
import { MatChipsModule } from '@angular/material/chips';
import { MatButtonModule } from '@angular/material/button';
import { MatAutocompleteModule } from '@angular/material/autocomplete';
import { RouterModule } from '@angular/router';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatInputModule } from '@angular/material/input';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';

const SettingType = {
    Boolean: 'boolean',
    String: 'string',
    Int: 'int',
    Double: 'double'
};

let FormHelper = class FormHelper {
    constructor(formBuilder) {
        this.formBuilder = formBuilder;
    }
    createSettingValuesFormGroup(settingValuesModel) {
        const formGroup = this.formBuilder.group({
            settingValues: this.formBuilder.array(settingValuesModel.settingValues.map(sv => {
                return this.createSettingValueFormGroup(sv, settingValuesModel.readOnly);
            })),
            // Decorator
            environment: this.formBuilder.group({
                environmentId: [{ value: settingValuesModel.environment.environmentId, disabled: true }],
                name: [{ value: settingValuesModel.environment.name, disabled: true }],
            }),
            comparisonAttributes: [[]],
            allRolloutRulesCount: [0]
        });
        const rolloutRules = this.getAllRolloutRules(formGroup);
        const comparisonAttributes = this.getComparisonAttributes(rolloutRules);
        formGroup.patchValue({ allRolloutRulesCount: rolloutRules.length });
        formGroup.patchValue({ comparisonAttributes: comparisonAttributes });
        return formGroup;
    }
    createSettingValueFormGroup(settingValueModel, readOnly) {
        return this.formBuilder.group({
            settingId: [settingValueModel.setting.settingId],
            value: [
                { value: settingValueModel.value, disabled: readOnly },
                this.getValidatorsForSettingType(settingValueModel.setting.settingType)
            ],
            rolloutPercentageItems: this.formBuilder.array(settingValueModel.rolloutPercentageItems.map(p => {
                return this.createRolloutPercentageItemFormGroup(p, readOnly, settingValueModel.setting.settingType);
            })),
            rolloutRules: this.formBuilder.array(settingValueModel.rolloutRules.map(r => {
                return this.createRolloutRuleFormGroup(r, readOnly, settingValueModel.setting.settingType);
            })),
            rolloutPercentageItemsSum: [{ value: 100, disabled: false }, [Validators.min(100), Validators.max(100)]],
            // Decoration
            setting: this.formBuilder.group({
                settingId: [{ value: settingValueModel.setting.settingId, disabled: true }],
                key: [{ value: settingValueModel.setting.key, disabled: true }],
                name: [{ value: settingValueModel.setting.name, disabled: true }],
                hint: [{ value: settingValueModel.setting.hint, disabled: true }],
                settingType: [{ value: settingValueModel.setting.settingType, disabled: true }]
            }),
            integrationLinks: this.formBuilder.array(settingValueModel.integrationLinks.map(il => {
                return this.formBuilder.group({
                    key: [{ value: il.key, disabled: true }],
                    description: [{ value: il.description, disabled: true }],
                    integrationLinkType: [{ value: il.integrationLinkType, disabled: true }],
                    url: [{ value: il.url, disabled: true }],
                });
            }))
        });
    }
    createRolloutPercentageItemFormGroup(p, readOnly, settingType) {
        return this.formBuilder.group({
            percentage: [{ value: p.percentage, disabled: readOnly }, [Validators.required, Validators.min(0), Validators.max(100)]],
            value: [
                { value: p.value, disabled: readOnly },
                [].concat(...[Validators.required, this.getValidatorsForSettingType(settingType)])
            ],
            valueDisabled: [
                { value: p.value, disabled: true },
            ]
        });
    }
    createRolloutRuleFormGroup(r, readOnly, settingType) {
        return this.formBuilder.group({
            comparisonAttribute: [{ value: r.comparisonAttribute, disabled: readOnly }, [Validators.required]],
            comparator: [{ value: r.comparator, disabled: readOnly }, [Validators.required]],
            comparisonValue: [{ value: r.comparisonValue, disabled: readOnly }, [Validators.required]],
            value: [
                { value: r.value, disabled: readOnly },
                this.getValidatorsForSettingType(settingType)
            ],
        });
    }
    getValidatorsForSettingType(settingType) {
        switch (settingType) {
            case SettingType.Int:
                return [Validators.required,
                    Validators.min(-2147483648), Validators.max(2147483647), Validators.pattern('^[-+]?[0-9]\\d*$')];
            case SettingType.Double:
                return [Validators.required, Validators.min(-1.79769313486232E+308), Validators.max(1.79769313486232E+308)];
            case SettingType.String:
            case SettingType.Boolean:
                return [Validators.required];
            default:
                return [];
        }
    }
    getAllRolloutRules(formGroup) {
        const settingValues = formGroup.get('settingValues');
        const rolloutRules = settingValues.controls.map(cv => cv.get('rolloutRules').value);
        return rolloutRules.length === 0 ? rolloutRules : rolloutRules.reduce((a, b) => a.concat(b));
    }
    getComparisonAttributes(rolloutRules) {
        if (rolloutRules.length > 0) {
            const currentComparisonAttributes = rolloutRules.map(r => r.comparisonAttribute);
            const defaultComparisonAttributes = ['Identifier', 'Email', 'Country'];
            const comparisonAttributes = currentComparisonAttributes.filter(c => defaultComparisonAttributes.indexOf(c) === -1);
            return Array.from(new Set(comparisonAttributes));
        }
        else {
            return [];
        }
    }
};
FormHelper.ctorParameters = () => [
    { type: FormBuilder }
];
FormHelper.ɵprov = ɵɵdefineInjectable({ factory: function FormHelper_Factory() { return new FormHelper(ɵɵinject(FormBuilder)); }, token: FormHelper, providedIn: "root" });
FormHelper = __decorate([
    Injectable({ providedIn: 'root' })
], FormHelper);

let ReasonDialogComponent = class ReasonDialogComponent {
    constructor(dialogRef, formBuilder) {
        this.dialogRef = dialogRef;
        this.formBuilder = formBuilder;
    }
    ngOnInit() {
        this.formGroup = this.formBuilder.group({
            reason: ['', [Validators.required, Validators.maxLength(1000)]]
        });
    }
    handleSaveButton() {
        if (!this.formGroup.valid) {
            return;
        }
        this.dialogRef.close({ button: 'save', reason: this.formGroup.controls.reason.value });
    }
    cancel() {
        this.dialogRef.close({ button: 'cancel' });
    }
};
ReasonDialogComponent.ctorParameters = () => [
    { type: MatDialogRef },
    { type: FormBuilder }
];
ReasonDialogComponent = __decorate([
    Component({
        selector: 'configcat-reason-dialog',
        template: "<h3 mat-dialog-title>Add notes</h3>\r\n<p>This will appear in the Why column of the Audit Log.</p>\r\n<form [formGroup]=\"formGroup\" (ngSubmit)=\"handleSaveButton()\">\r\n  <div mat-dialog-content class=\"container\">\r\n    <mat-form-field appearance=\"outline\">\r\n      <mat-label></mat-label>\r\n      <textarea matInput formControlName=\"reason\" name=\"reason\" placeholder=\"I'm changing settings because...\"\r\n        rows=\"3\"></textarea>\r\n      <mat-error *ngIf=\"formGroup.controls.reason.invalid\">The field is required.</mat-error>\r\n    </mat-form-field>\r\n  </div>\r\n  <br>\r\n  <div mat-dialog-actions>\r\n    <button type=\"submit\" style=\"margin-right: 10px;\" mat-raised-button color=\"primary\">Add notes & Save</button>\r\n    <button type=\"button\" mat-raised-button (click)=\"cancel()\">Cancel</button>\r\n  </div>\r\n</form>",
        styles: [".container>*{width:100%}"]
    })
], ReasonDialogComponent);

let SaveSnackbarComponent = class SaveSnackbarComponent {
    constructor() { }
    ngOnInit() {
    }
};
SaveSnackbarComponent = __decorate([
    Component({
        selector: 'configcat-save-snackbar',
        template: "<p class=\"header\">Settings saved successfully.</p>",
        styles: ["p{text-align:center;font-weight:700}.header{font-size:1.2em}"]
    })
], SaveSnackbarComponent);

let ConcurrentDialogComponent = class ConcurrentDialogComponent {
    constructor(dialogRef) {
        this.dialogRef = dialogRef;
    }
    ngOnInit() {
    }
    save() {
        this.dialogRef.close({ button: 'save' });
    }
    discard() {
        this.dialogRef.close({ button: 'discard' });
    }
};
ConcurrentDialogComponent.ctorParameters = () => [
    { type: MatDialogRef }
];
ConcurrentDialogComponent = __decorate([
    Component({
        selector: 'configcat-concurrent-dialog',
        template: "<h3 mat-dialog-title>Concurrent update detected</h3>\r\n<p>\r\n    Somebody updated the feature flags & settings while you were editing them.\r\n</p>\r\n<ul>\r\n    <li>\r\n        To discard your local changes and reload the updated values, click <b>Discard my changes & reload</b>\r\n    </li>\r\n    <li>\r\n        To save you local values and overwrite the remote changes, click <b>Save anyway</b>\r\n    </li>\r\n</ul>\r\n<p>For more info about the changes check the <a [routerLink]=\"['/auditlog']\" target=\"_blank\" >audit logs</a> in a new tab.</p>\r\n<div mat-dialog-actions>\r\n    <button type=\"button\" style=\"margin-right: 10px;\" mat-raised-button color=\"primary\" (click)=\"discard()\">Discard\r\n        local changes & reload</button>\r\n    <button type=\"button\" mat-raised-button (click)=\"save()\">Save anyway</button>\r\n</div>",
        styles: [""]
    })
], ConcurrentDialogComponent);

let SettingValuesComponent = class SettingValuesComponent {
    constructor(formHelper, snackBar, dialog) {
        this.formHelper = formHelper;
        this.snackBar = snackBar;
        this.dialog = dialog;
        this.showVariationId = false;
        this.deleteSettingText = 'Delete setting';
        this.showIntegrationLinks = false;
        this.showEnvironmentName = false;
        this.subscriptionLimitReached = new EventEmitter();
        this.formStatusChanged = new EventEmitter();
        this.editSettingRequested = new EventEmitter();
        this.deleteSettingRequested = new EventEmitter();
        this.formValuesChanged = new EventEmitter();
        this.loading = true;
        this.submitting = false;
        this.showRevertQuestion = false;
        this.showFeedBackMessage = false;
        this.readOnly = false;
    }
    ngOnInit() {
        this.reloadSettingValues();
    }
    ngOnDestroy() {
        if (this.valueChangesSubscription) {
            this.valueChangesSubscription.unsubscribe();
            this.valueChangesSubscription = null;
        }
    }
    reloadSettingValues() {
        this.showFeedBackMessage = false;
        this.loading = true;
        this.loadSettingValues()
            .subscribe(settingValues => {
            this.readOnly = settingValues.readOnly;
            this.settingValues = settingValues;
            this.createFormGroup();
            this.loading = false;
        }, error => {
            this.snackBar.open('Ooops. We could not load your Feature Flags and Settings. Please try again or contact us.', 'Dismiss', { duration: 60000, panelClass: 'red-snackbar' });
        });
    }
    createFormGroup() {
        if (this.valueChangesSubscription) {
            this.valueChangesSubscription.unsubscribe();
            this.valueChangesSubscription = null;
        }
        this.formGroup = this.formHelper.createSettingValuesFormGroup(this.settingValues);
        this.formStatusChanged.emit({ dirty: false });
        this.formGroup.valueChanges.subscribe(() => {
            this.formStatusChanged.emit({ dirty: this.formGroup.dirty });
            this.formValuesChanged.emit();
        });
    }
    settingValuesFormArray() {
        return this.formGroup.get('settingValues');
    }
    onSubscriptionLimitReached() {
        this.subscriptionLimitReached.emit();
    }
    submit() {
        if (!this.formGroup || !this.formGroup.valid) {
            this.showFeedBackMessage = true;
            this.formGroup.markAsTouched();
            return;
        }
        this.showFeedBackMessage = false;
        this.submitting = true;
        const updateSettingValuesModel = {
            settingValues: this.formGroup.controls.settingValues.value,
            knownLastUpdatedAt: this.settingValues.lastUpdatedAt,
            forced: false
        };
        if (this.settingValues.reasonRequired) {
            const dialogRef = this.dialog.open(ReasonDialogComponent, {
                width: '450px'
            });
            dialogRef.afterClosed()
                .subscribe(result => {
                if (result && result.button === 'save' && result.reason) {
                    updateSettingValuesModel.reason = result.reason;
                    this.updateSettingValues(updateSettingValuesModel);
                }
                else {
                    this.submitting = false;
                }
            });
            return;
        }
        this.updateSettingValues(updateSettingValuesModel);
    }
    updateSettingValues(updateSettingValuesModel) {
        this.onSave(updateSettingValuesModel).subscribe(settingValues => {
            this.settingValues = settingValues;
            this.createFormGroup();
            this.submitting = false;
            this.snackBar.openFromComponent(SaveSnackbarComponent, { duration: 5000, panelClass: 'green-snackbar' });
        }, (error) => {
            if (error.status === 400 && error.error
                && error.error.KnownLastUpdatedAt && error.error.KnownLastUpdatedAt[0] === 'Concurrent update') {
                const dialogRef = this.dialog.open(ConcurrentDialogComponent, {
                    width: '560px'
                });
                dialogRef.afterClosed()
                    .subscribe(result => {
                    if (result && result.button === 'save') {
                        updateSettingValuesModel.forced = true;
                        this.updateSettingValues(updateSettingValuesModel);
                    }
                    else {
                        this.reloadSettingValues();
                        this.submitting = false;
                    }
                });
                return;
            }
            if (error.status === 400 && error.error && error.error.Reason && error.error.Reason[0] === 'Reason required') {
                const dialogRef = this.dialog.open(ReasonDialogComponent, {
                    width: '450px'
                });
                dialogRef.afterClosed()
                    .subscribe(result => {
                    if (result && result.button === 'save' && result.reason) {
                        updateSettingValuesModel.reason = result.reason;
                        this.updateSettingValues(updateSettingValuesModel);
                    }
                    else {
                        this.submitting = false;
                    }
                });
                return;
            }
            this.formGroup.setErrors({ internalServerError: 'Internal server error' });
            this.showFeedBackMessage = true;
        });
    }
    handleRevertYes() {
        this.showRevertQuestion = false;
        this.reloadSettingValues();
    }
    handleRevertCancel() {
        this.showRevertQuestion = false;
    }
    onEditSettingRequested(setting) {
        this.editSettingRequested.emit({
            setting: setting.setting,
            environment: this.settingValues.environment,
            config: this.settingValues.config
        });
    }
    onDeleteSettingRequested(setting) {
        this.deleteSettingRequested.emit({
            setting: setting.setting,
            environment: this.settingValues.environment,
            config: this.settingValues.config
        });
    }
};
SettingValuesComponent.ctorParameters = () => [
    { type: FormHelper },
    { type: MatSnackBar },
    { type: MatDialog }
];
__decorate([
    Input()
], SettingValuesComponent.prototype, "canEditSetting", void 0);
__decorate([
    Input()
], SettingValuesComponent.prototype, "canDeleteSetting", void 0);
__decorate([
    Input()
], SettingValuesComponent.prototype, "showVariationId", void 0);
__decorate([
    Input()
], SettingValuesComponent.prototype, "onSave", void 0);
__decorate([
    Input()
], SettingValuesComponent.prototype, "loadSettingValues", void 0);
__decorate([
    Input()
], SettingValuesComponent.prototype, "deleteSettingText", void 0);
__decorate([
    Input()
], SettingValuesComponent.prototype, "showIntegrationLinks", void 0);
__decorate([
    Input()
], SettingValuesComponent.prototype, "showEnvironmentName", void 0);
__decorate([
    Output()
], SettingValuesComponent.prototype, "subscriptionLimitReached", void 0);
__decorate([
    Output()
], SettingValuesComponent.prototype, "formStatusChanged", void 0);
__decorate([
    Output()
], SettingValuesComponent.prototype, "editSettingRequested", void 0);
__decorate([
    Output()
], SettingValuesComponent.prototype, "deleteSettingRequested", void 0);
__decorate([
    Output()
], SettingValuesComponent.prototype, "formValuesChanged", void 0);
SettingValuesComponent = __decorate([
    Component({
        selector: 'configcat-setting-values',
        template: "<form [formGroup]=\"formGroup\" (ngSubmit)=\"submit()\" *ngIf=\"!loading\">\r\n    <div class=\"wrapper\" *ngFor=\"let settingValueFromGroup of settingValuesFormArray().controls; let i = index\">\r\n        <configcat-setting-value [formGroup]=\"settingValueFromGroup\" [readOnly]=\"readOnly\"\r\n            [canEditSetting]=\"canEditSetting\" [canDeleteSetting]=\"canDeleteSetting\" [showVariationId]=\"showVariationId\"\r\n            (subscriptionLimitReached)=\"onSubscriptionLimitReached()\"\r\n            [rolloutLimitations]=\"settingValues.rolloutLimitations\"\r\n            (editSettingRequested)=\"onEditSettingRequested($event)\"\r\n            (deleteSettingRequested)=\"onDeleteSettingRequested($event)\" [deleteSettingText]=\"deleteSettingText\"\r\n            [showIntegrationLinks]=\"showIntegrationLinks\" [showEnvironmentName]=\"showEnvironmentName\">\r\n        </configcat-setting-value>\r\n    </div>\r\n\r\n    <div class=\"save\" *ngIf=\"!readOnly && formGroup.dirty\">\r\n        <button class=\"save-button\" mat-flat-button color=\"primary\" type=\"submit\" [disabled]=\"submitting\">\r\n            <mat-icon>cloud</mat-icon> <span> Save changes</span>\r\n        </button>\r\n\r\n        <div *ngIf=\"!submitting && formGroup.dirty\">\r\n            <button type=\"button\" mat-stroked-button *ngIf=\"!showRevertQuestion\" (click)=\"showRevertQuestion = true\">\r\n                <mat-icon>history</mat-icon><span>Revert changes</span>\r\n            </button>\r\n            <div *ngIf=\"showRevertQuestion\">\r\n                <span class=\"revert\">Revert to last saved state?</span>\r\n                <button type=\"button\" class=\"yes\" mat-stroked-button (click)=\"handleRevertYes()\">Yes</button>\r\n                <button type=\"button\" mat-stroked-button (click)=\"handleRevertCancel()\">No</button>\r\n            </div>\r\n        </div>\r\n\r\n        <div class=\"feedback\" *ngIf=\"formGroup.touched && formGroup.invalid && showFeedBackMessage\">\r\n            <div class=\"error\" *ngIf=\"formGroup.errors && formGroup.errors.internalServerError\">\r\n                <span>Something went wrong and we could not save your settings. You can try again or <a\r\n                        href=\"https://configcat.com/support\" target=\"_blank\">contact\r\n                        us.</a></span>\r\n            </div>\r\n            <div class=\"error\" *ngIf=\"!formGroup.errors || !formGroup.errors.internalServerError\">\r\n                <span>Oh no! A feature flag has invalid or missing data.</span>\r\n            </div>\r\n        </div>\r\n    </div>\r\n</form>",
        styles: [".wrapper{background-color:#f3f3f3;border:1px solid rgba(0,0,0,.12);border-radius:4px;margin:1em 0}.save{position:-webkit-sticky;position:sticky;bottom:0;min-height:38px;padding:.5em 0;background-color:#fff;display:flex;flex-direction:row;flex-wrap:wrap;align-items:center}.save .save-button{-webkit-animation:1s 3 shadow-pulse;animation:1s 3 shadow-pulse}.save>*{margin:.3em}.save .revert{margin:0 16px;font-size:14px}.save .yes{margin:0 5px}.error{background-color:#a32b2d;color:#fff;padding:5px;font-size:.9em}.error a{color:#fff;text-decoration:underline}"]
    })
], SettingValuesComponent);

const RolloutRuleComparator = {
    IsOneOf: 'isOneOf',
    IsNotOneOf: 'isNotOneOf',
    Contains: 'contains',
    DoesNotContain: 'doesNotContain',
    SemVerIsOneOf: 'semVerIsOneOf',
    SemVerIsNotOneOf: 'semVerIsNotOneOf',
    SemVerLess: 'semVerLess',
    SemVerLessOrEquals: 'semVerLessOrEquals',
    SemVerGreater: 'semVerGreater',
    SemVerGreaterOrEquals: 'semVerGreaterOrEquals',
    NumberEquals: 'numberEquals',
    NumberDoesNotEqual: 'numberDoesNotEqual',
    NumberLess: 'numberLess',
    NumberLessOrEquals: 'numberLessOrEquals',
    NumberGreater: 'numberGreater',
    NumberGreaterOrEquals: 'numberGreaterOrEquals',
    SensitiveIsOneOf: 'sensitiveIsOneOf',
    SensitiveIsNotOneOf: 'sensitiveIsNotOneOf'
};

const IntegrationLinkType = {
    Trello: 'trello',
    Jira: 'jira'
};

let SettingValueComponent = class SettingValueComponent {
    constructor(formHelper, cdRef) {
        this.formHelper = formHelper;
        this.cdRef = cdRef;
        this.showVariationId = false;
        this.deleteSettingText = 'Delete setting';
        this.showIntegrationLinks = false;
        this.showEnvironmentName = false;
        this.subscriptionLimitReached = new EventEmitter();
        this.editSettingRequested = new EventEmitter();
        this.deleteSettingRequested = new EventEmitter();
        this.areAllRulesVisible = true;
        this.rulesBeingRendered = false;
        this.SettingType = SettingType;
    }
    ngOnInit() {
        this.areAllRulesVisible = this.allRolloutRulesCount() <= 20 && this.rolloutRules().controls.length <= 10;
    }
    allRolloutRulesCount() {
        return this.formGroup.root.get('allRolloutRulesCount').value;
    }
    environment() {
        return this.formGroup.root.get('environment').value;
    }
    setting() {
        return this.formGroup.get('setting').value;
    }
    integrationLinks() {
        return this.formGroup.get('integrationLinks');
    }
    rolloutRules() {
        return this.formGroup.get('rolloutRules');
    }
    rolloutPercentageItems() {
        return this.formGroup.get('rolloutPercentageItems');
    }
    value() {
        return this.formGroup.get('value');
    }
    addRolloutRule() {
        this.showAllRules();
        if (this.rolloutLimitations && this.rolloutRules().length >= this.rolloutLimitations.maxRolloutRuleCount) {
            this.subscriptionLimitReached.emit();
            return;
        }
        this.rolloutRules().push(this.formHelper.createRolloutRuleFormGroup({
            comparisonAttribute: 'Identifier',
            comparator: RolloutRuleComparator.IsOneOf,
            comparisonValue: '',
            value: this.setting().settingType === SettingType.Boolean ? false : null
        }, this.readOnly, this.setting().settingType));
        this.rolloutRules().markAsDirty();
    }
    removeRolloutRules() {
        this.rolloutRules().clear();
        this.rolloutRules().markAsDirty();
    }
    addRolloutPercentageItem() {
        if (this.rolloutLimitations && this.rolloutPercentageItems().length >= this.rolloutLimitations.maxRolloutPercentageItemCount) {
            this.subscriptionLimitReached.emit();
            return;
        }
        if (this.rolloutPercentageItems().length === 0) {
            switch (this.setting().settingType) {
                case SettingType.Boolean:
                    this.rolloutPercentageItems().push(this.formHelper.createRolloutPercentageItemFormGroup({
                        percentage: 0,
                        value: true
                    }, this.readOnly, this.setting().settingType));
                    this.rolloutPercentageItems().push(this.formHelper.createRolloutPercentageItemFormGroup({
                        percentage: 100,
                        value: false
                    }, this.readOnly, this.setting().settingType));
                    break;
                case SettingType.String:
                case SettingType.Int:
                case SettingType.Double:
                    for (let i = 0; i < 4; i++) {
                        this.rolloutPercentageItems().push(this.formHelper.createRolloutPercentageItemFormGroup({
                            percentage: 25,
                            value: null
                        }, this.readOnly, this.setting().settingType));
                    }
                    break;
            }
        }
        else if (this.setting().settingType !== SettingType.Boolean) {
            this.rolloutPercentageItems().push(this.formHelper.createRolloutPercentageItemFormGroup({
                percentage: 0,
                value: null
            }, this.readOnly, this.setting().settingType));
        }
        this.rolloutPercentageItems().markAsDirty();
    }
    removeRolloutPercentageItems() {
        this.rolloutPercentageItems().clear();
        this.rolloutPercentageItems().markAsDirty();
    }
    editSetting() {
        this.editSettingRequested.emit({ setting: this.setting() });
    }
    deleteSetting() {
        this.deleteSettingRequested.emit({ setting: this.setting() });
    }
    showAllRules() {
        this.rulesBeingRendered = true;
        setTimeout(() => {
            this.cdRef.detach();
            this.areAllRulesVisible = true;
            this.cdRef.detectChanges();
            this.rulesBeingRendered = false;
            this.cdRef.reattach();
        }, 10);
    }
};
SettingValueComponent.ctorParameters = () => [
    { type: FormHelper },
    { type: ChangeDetectorRef }
];
__decorate([
    Input()
], SettingValueComponent.prototype, "formGroup", void 0);
__decorate([
    Input()
], SettingValueComponent.prototype, "readOnly", void 0);
__decorate([
    Input()
], SettingValueComponent.prototype, "canEditSetting", void 0);
__decorate([
    Input()
], SettingValueComponent.prototype, "canDeleteSetting", void 0);
__decorate([
    Input()
], SettingValueComponent.prototype, "rolloutLimitations", void 0);
__decorate([
    Input()
], SettingValueComponent.prototype, "showVariationId", void 0);
__decorate([
    Input()
], SettingValueComponent.prototype, "deleteSettingText", void 0);
__decorate([
    Input()
], SettingValueComponent.prototype, "showIntegrationLinks", void 0);
__decorate([
    Input()
], SettingValueComponent.prototype, "showEnvironmentName", void 0);
__decorate([
    Output()
], SettingValueComponent.prototype, "subscriptionLimitReached", void 0);
__decorate([
    Output()
], SettingValueComponent.prototype, "editSettingRequested", void 0);
__decorate([
    Output()
], SettingValueComponent.prototype, "deleteSettingRequested", void 0);
SettingValueComponent = __decorate([
    Component({
        selector: 'configcat-setting-value',
        template: "<div class=\"setting-full\" [formGroup]=\"formGroup\">\r\n    <div class=\"delete\" (click)=\"deleteSetting()\" *ngIf=\"canDeleteSetting\" [matTooltip]=\"deleteSettingText\"\r\n        data-robot=\"deleteSetting\">\r\n        <mat-icon>cancel</mat-icon>\r\n    </div>\r\n    <div class=\"setting-details\">\r\n        <div class=\"setting-name flex\">\r\n            <div class=\"name\" data-robot=\"settingName\">\r\n                {{setting().name}}\r\n                <span *ngIf=\"showEnvironmentName\"> ({{environment().name}})</span>\r\n            </div>\r\n            <div class=\"edit\" matTooltip=\"Edit name and description\" (click)=\"editSetting()\" *ngIf=\"canEditSetting\"\r\n                data-robot=\"renameSetting\">\r\n                <mat-icon>edit</mat-icon>\r\n            </div>\r\n        </div>\r\n        <div class=\"setting-key\"><code data-robot=\"settingKey\">{{setting().key}}</code></div>\r\n        <span *ngIf=\"setting().hint\" class=\"description\">{{setting().hint}}</span>\r\n\r\n        <mat-chip-list *ngIf=\"showIntegrationLinks\">\r\n            <mat-chip *ngFor=\"let integrationLink of integrationLinks().controls\">\r\n                <a href=\"{{integrationLink.value.url}}\" target=\"_blank\">{{integrationLink.value.description}}</a>\r\n                <mat-icon matChipRemove>cancel</mat-icon>\r\n            </mat-chip>\r\n        </mat-chip-list>\r\n    </div>\r\n\r\n    <div class=\"rules-list\">\r\n        <div class=\"show-hide-toggle\" *ngIf=\"allRolloutRulesCount() > 20 || rolloutRules().controls.length > 10\">\r\n            <a *ngIf=\"!areAllRulesVisible && !rulesBeingRendered && rolloutRules().controls.length > 0\"\r\n                (click)=\"showAllRules()\">\r\n                Show {{rolloutRules().controls.length}} hidden targeting rules\r\n            </a>\r\n            <a *ngIf=\"areAllRulesVisible\" (click)=\"areAllRulesVisible = false\">Hide targeting rules\r\n            </a>\r\n        </div>\r\n\r\n        <div *ngIf=\"this.areAllRulesVisible\">\r\n            <configcat-rollout-rules *ngIf=\"rolloutRules().controls.length !== 0\" [formGroup]=\"formGroup\"\r\n                [maxRolloutRuleCount]=\"rolloutLimitations?.maxRolloutRuleCount\" [showVariationId]=\"showVariationId\">\r\n            </configcat-rollout-rules>\r\n        </div>\r\n\r\n        <configcat-rollout-percentage-items *ngIf=\"rolloutPercentageItems().controls.length !== 0\"\r\n            [formGroup]=\"formGroup\" [maxRolloutPercentageItemCount]=\"rolloutLimitations?.maxRolloutPercentageItemCount\"\r\n            [showVariationId]=\"showVariationId\">\r\n        </configcat-rollout-percentage-items>\r\n\r\n        <div class=\"all-other flex\">\r\n            <div *ngIf=\"rolloutPercentageItems().length === 0 && rolloutRules().length !== 0\"\r\n                data-tooltip=\"This value will be served to all users who the above rules don't apply.\">\r\n                <mat-chip-list>\r\n                    <mat-chip class=\"chip\">To all other users, serve:</mat-chip>\r\n                </mat-chip-list>\r\n            </div>\r\n            <div *ngIf=\"rolloutPercentageItems().length !== 0\"\r\n                data-tooltip=\"This fallback value will be served in case the user can't be identified.\">\r\n                <mat-chip-list>\r\n                    <mat-chip class=\"chip\">To unidentified users, serve:</mat-chip>\r\n                </mat-chip-list>\r\n            </div>\r\n            <div *ngIf=\"rolloutPercentageItems().length === 0 && rolloutRules().length === 0\">\r\n                <mat-chip-list>\r\n                    <mat-chip class=\"chip\">To all users, serve:</mat-chip>\r\n                </mat-chip-list>\r\n            </div>\r\n            <configcat-value class=\"served-value\" [valueFormControl]=\"value()\" [setting]=\"setting()\"\r\n                [showVariationId]=\"showVariationId\">\r\n            </configcat-value>\r\n        </div>\r\n        <button type=\"button\" class=\"sige-button targeting-button\" (click)=\"addRolloutRule()\" *ngIf=\"!readOnly\">\r\n            {{rolloutRules().length === 0 ? 'Target specific users' : 'Add new targeting rule'}}\r\n        </button>\r\n        <button type=\"button\" class=\"sige-button targeting-button\" (click)=\"addRolloutPercentageItem()\"\r\n            *ngIf=\"!readOnly\"\r\n            [disabled]=\"setting().settingType === SettingType.Boolean && rolloutPercentageItems().length !== 0\">\r\n            {{rolloutPercentageItems().length === 0 ? 'Target % of users' : 'Add new % option'}}\r\n        </button>\r\n    </div>\r\n</div>",
        styles: [".setting-full{position:relative;display:flex;flex-wrap:nowrap}.setting-full .delete{position:absolute;right:-15px;top:-10px;color:#d6a9a9;cursor:pointer}.setting-full .delete:hover{color:#ed5251}.setting-full .delete mat-icon{font-size:20px}.setting-full .setting-details{display:flex;flex-direction:column;width:400px;padding:1em 0 1em 1em}.setting-full .setting-details>*{margin:.5em}.setting-full .setting-details .setting-name{flex-direction:row}.setting-full .setting-details .setting-name:hover>.edit{visibility:visible}.setting-full .setting-details .setting-name .name{font-weight:700;word-wrap:break-word;max-width:100%}.setting-full .setting-details .setting-name .edit{margin:5px 0 0 5px;cursor:pointer;visibility:hidden;color:grey}.setting-full .setting-details .setting-name .edit:hover{visibility:visible}.setting-full .setting-details .setting-name .edit mat-icon{font-size:18px}.setting-full .setting-details .setting-key{word-wrap:break-word}.setting-full .setting-details .description{font-size:.8em}.setting-full .setting-details code{background-color:#d3d3d3;padding:3px 6px}.setting-full .rules-list{width:100%;padding:1em}.setting-full .targeting-button{margin-left:1rem;margin-bottom:1em}.all-other{min-height:40px;justify-content:flex-start;border:1px solid #d5d5d5;border-radius:4px;background-color:#fff;margin:.5em 1em;padding:4px 0 4px .5em}.all-other .chip{background-color:#67b1fc3d}.all-other .served-value{margin-left:auto;min-width:100px}.end{margin-left:auto}.flex{display:flex;flex-wrap:wrap;align-items:center}.show-hide-toggle{margin-left:1em;cursor:pointer}.sige-button{background-color:#d9eadc;color:#363636;cursor:pointer;justify-content:center;padding:.375em .75em;text-align:center;white-space:nowrap;border-radius:2px;font-size:.75rem;border:1px solid #dbdbdb;box-shadow:none;display:inline-flex;font-family:Montserrat,Roboto,\"Helvetica Neue\",sans-serif}.sige-button:hover{background-color:#bad0be}.sige-button:disabled{background-color:#fff;color:#b3b3b3}@media (max-width:1200px){.setting-full .setting-details{width:300px}}@media (max-width:700px){.setting-full{flex-wrap:wrap}.setting-full .setting-details{padding:.2em 0 .2em .2em}.setting-full .all-other{margin:.5em .3em}}"]
    })
], SettingValueComponent);

let RolloutPercentageItemsComponent = class RolloutPercentageItemsComponent {
    constructor() {
        this.SettingType = SettingType;
    }
    setting() {
        return this.formGroup.get('setting').value;
    }
    rolloutRules() {
        return this.formGroup.get('rolloutRules');
    }
    rolloutPercentageItems() {
        return this.formGroup.get('rolloutPercentageItems');
    }
    rolloutPercentageItemsSum() {
        return this.formGroup.get('rolloutPercentageItemsSum');
    }
    value(rolloutPercentageItem) {
        return this.setting().settingType === SettingType.Boolean ?
            rolloutPercentageItem.get('valueDisabled') :
            rolloutPercentageItem.get('value');
    }
    percentageChanged(index) {
        if (this.rolloutPercentageItems().controls[index].value.percentage < 0) {
            this.rolloutPercentageItems().controls[index].controls.percentage.setValue(0);
        }
        if (this.rolloutPercentageItems().controls[index].value.percentage > 100) {
            this.rolloutPercentageItems().controls[index].controls.percentage.setValue(100);
        }
        if (this.rolloutPercentageItems().controls.length === 2) {
            this.rolloutPercentageItems().controls[1 - index].controls.percentage
                .setValue(100 - this.rolloutPercentageItems().controls[index].value.percentage);
        }
        this.updateSum();
    }
    updateSum() {
        if (this.rolloutPercentageItems().controls.length === 0) {
            this.rolloutPercentageItemsSum().setValue(100);
        }
        else {
            this.rolloutPercentageItemsSum().setValue(this.rolloutPercentageItems().controls.map(v => v.value.percentage)
                .reduce((a, b) => a + b));
        }
    }
    removeRolloutPercentageItem(index) {
        this.rolloutPercentageItems().removeAt(index);
        this.rolloutPercentageItems().markAsDirty();
        this.updateSum();
    }
    removeAll() {
        this.rolloutPercentageItems().clear();
        this.rolloutPercentageItems().markAsDirty();
        this.updateSum();
    }
};
__decorate([
    Input()
], RolloutPercentageItemsComponent.prototype, "formGroup", void 0);
__decorate([
    Input()
], RolloutPercentageItemsComponent.prototype, "readOnly", void 0);
__decorate([
    Input()
], RolloutPercentageItemsComponent.prototype, "maxRolloutPercentageItemCount", void 0);
__decorate([
    Input()
], RolloutPercentageItemsComponent.prototype, "showVariationId", void 0);
RolloutPercentageItemsComponent = __decorate([
    Component({
        selector: 'configcat-rollout-percentage-items',
        template: "<div class=\"percentage-wrapper\" [formGroup]=\"formGroup\">\r\n\r\n    <div class=\"if\" *ngIf=\"rolloutRules().controls.length > 0\">\r\n        <mat-chip-list>\r\n            <mat-chip class=\"chip\">ELSE IF</mat-chip>\r\n        </mat-chip-list>\r\n    </div>\r\n    <div class=\"grow\">\r\n        <div class=\"percentage-row flex\"\r\n            *ngFor=\"let rolloutPercentageItem of rolloutPercentageItems().controls; let i = index\">\r\n            <div class=\"flex\">\r\n                <input type=\"number\" [formControl]=\"rolloutPercentageItem.controls.percentage\"\r\n                    (input)=\"percentageChanged(i)\">%\r\n                <span class=\"users\">of <a href=\"https://configcat.com/docs/advanced/user-object\"\r\n                        target=\"_blank\">users</a>\r\n                </span>\r\n            </div>\r\n\r\n            <div class=\"delete-option\" matTooltip=\"Remove this % option\"\r\n                *ngIf=\"!readOnly && rolloutPercentageItems().controls.length > 2\"\r\n                (click)=\"removeRolloutPercentageItem(i)\">\r\n                <mat-icon>cancel</mat-icon>\r\n            </div>\r\n\r\n            <span data-tooltip=\"The sum of percentages must be equal to 100.\"\r\n                *ngIf=\"(i === rolloutPercentageItems().controls.length -1) && (rolloutPercentageItemsSum().invalid)\">\r\n                <mat-icon color=\"warn\">\r\n                    error_outline\r\n                </mat-icon>\r\n            </span>\r\n\r\n            <div class=\"value flex\">\r\n                <span class=\"value-name\" *ngIf=\"setting().settingType !== SettingType.Boolean\">Served value:</span>\r\n                <configcat-value class=\"served-value\" [valueFormControl]=\"value(rolloutPercentageItem)\" [setting]=\"setting()\"\r\n                    [showVariationId]=\"showVariationId\">\r\n                </configcat-value>\r\n            </div>\r\n            <div class=\"delete\" matTooltip=\"Remove % options\" (click)=\"removeAll()\" *ngIf=\"!readOnly\">\r\n                <mat-icon>cancel</mat-icon>\r\n            </div>\r\n        </div>\r\n    </div>\r\n</div>",
        styles: [".percentage-wrapper{border:1px solid #d5d5d5;background-color:#fff;border-radius:4px;margin:.5em 1em;position:relative;padding:.5em 0;display:flex}.percentage-wrapper .delete{position:absolute;color:#d6a9a9;cursor:pointer;top:-8px;right:-12px}.percentage-wrapper .delete:hover{color:#ed5251}.percentage-wrapper .delete mat-icon{font-size:20px}.percentage-wrapper .delete-option{color:#d6a9a9;cursor:pointer;margin-left:1rem;margin-top:4px}.percentage-wrapper .delete-option:hover{color:#ed5251}.percentage-wrapper .delete-option mat-icon{font-size:20px}.if{display:flex;align-items:center}.if>*{margin:0 .5em}.if .spacing{margin-right:44px}.if .chip{background-color:#67b1fc3d}.grow{flex-grow:1}.percentage-row{min-height:40px;margin-left:.5em}.flex{display:flex;flex-wrap:wrap;align-items:center}.value{margin-left:auto;min-width:100px}.value .value-name{display:none}input{background-color:#fff;border:1px solid #e5e5e5;min-height:28px;width:40px;margin-right:5px;font-size:15px;padding:0 5px;font-family:Montserrat,Roboto,\"Helvetica Neue\",sans-serif;font-weight:300}.users{margin-left:20px}@media (max-width:700px){.percentage-wrapper{margin:.5em .3em}}"]
    })
], RolloutPercentageItemsComponent);

let ExpandDialogComponent = class ExpandDialogComponent {
    constructor(dialogRef, formBuilder, data) {
        this.dialogRef = dialogRef;
        this.formBuilder = formBuilder;
        this.data = data;
    }
    ngOnInit() {
        if (this.data.multiline) {
            this.buildFormGroup();
        }
        else {
            this.formControlSubscription = this.data.control.valueChanges.subscribe(value => {
                this.data.control.setValue(value, { onlySelf: true, emitEvent: false, emitModelToViewChange: true });
                this.data.control.markAsDirty();
            }, error => { }, () => { });
        }
    }
    values() {
        return this.formGroup.get('values');
    }
    buildFormGroup() {
        const value = this.data.control.value ? this.data.control.value : '';
        const list = value.split(',').map(v => v.trim()).filter(v => !!v);
        this.formGroup = this.formBuilder.group({
            values: this.formBuilder.array(list.map(v => this.formBuilder.control(v)))
        });
        this.formGroupSubscription = this.formGroup.valueChanges.subscribe((formGroupValue) => {
            this.data.control.setValue(formGroupValue.values
                .filter(v => !!v)
                .join(','), { onlySelf: true, emitEvent: false, emitModelToViewChange: true });
            this.data.control.updateValueAndValidity();
            this.data.control.markAsDirty();
        });
    }
    add() {
        this.formGroup.controls.values.push(this.formBuilder.control(''));
    }
    remove(index) {
        this.formGroup.controls.values.removeAt(index);
    }
    ngOnDestroy() {
        if (this.formControlSubscription) {
            this.formControlSubscription.unsubscribe();
            this.formControlSubscription = null;
        }
        if (this.formGroupSubscription) {
            this.formGroupSubscription.unsubscribe();
            this.formGroupSubscription = null;
        }
    }
    close() {
        this.dialogRef.close();
    }
};
ExpandDialogComponent.ctorParameters = () => [
    { type: MatDialogRef },
    { type: FormBuilder },
    { type: undefined, decorators: [{ type: Inject, args: [MAT_DIALOG_DATA,] }] }
];
ExpandDialogComponent = __decorate([
    Component({
        selector: 'configcat-expand-dialog',
        template: "<div mat-dialog-content class=\"container\">\r\n\r\n    <mat-form-field appearance=\"outline\" *ngIf=\"!data.multiline\">\r\n        <mat-label></mat-label>\r\n        <textarea matInput [formControl]=\"data.control\" rows=\"15\"></textarea>\r\n    </mat-form-field>\r\n    <div [formGroup]=\"formGroup\" *ngIf=\"data.multiline\">\r\n        <div *ngFor=\"let valueControl of values().controls; let i = index;\" class=\"value-field\">\r\n            <mat-form-field appearance=\"outline\">\r\n                <input matInput [formControl]=\"valueControl\" rows=\"15\" />\r\n            </mat-form-field>\r\n            <button (click)=\"remove(i)\" mat-icon-button type=\"button\">\r\n                <mat-icon>delete_outline</mat-icon>\r\n            </button>\r\n        </div>\r\n        <button (click)=\"add()\" mat-icon-button type=\"button\">\r\n            <mat-icon>add</mat-icon>\r\n        </button>\r\n    </div>\r\n</div>\r\n<div mat-dialog-actions>\r\n    <button type=\"button\" mat-raised-button color=\"primary\" (click)=\"close()\">Done</button>\r\n</div>",
        styles: [".container>*{width:100%}.container>* .value-field{display:flex;font-size:12px;flex-wrap:nowrap}.container>* .value-field mat-form-field{flex-grow:1}"]
    }),
    __param(2, Inject(MAT_DIALOG_DATA))
], ExpandDialogComponent);

let RolloutRulesComponent = class RolloutRulesComponent {
    constructor(dialog) {
        this.dialog = dialog;
        this.SettingType = SettingType;
        this.defaultComparisonAttributes = ['Identifier', 'Email', 'Country'];
        this.rolloutRuleComparators = [
            { value: RolloutRuleComparator.IsOneOf, label: 'IS ONE OF' },
            { value: RolloutRuleComparator.IsNotOneOf, label: 'IS NOT ONE OF' },
            { value: RolloutRuleComparator.Contains, label: 'CONTAINS' },
            { value: RolloutRuleComparator.DoesNotContain, label: 'DOES NOT CONTAIN' },
            { value: RolloutRuleComparator.SemVerIsOneOf, label: 'IS ONE OF (SemVer)' },
            { value: RolloutRuleComparator.SemVerIsNotOneOf, label: 'IS NOT ONE OF (SemVer)' },
            { value: RolloutRuleComparator.SemVerLess, label: '< (SemVer)' },
            { value: RolloutRuleComparator.SemVerLessOrEquals, label: '<= (SemVer)' },
            { value: RolloutRuleComparator.SemVerGreater, label: '> (SemVer)' },
            { value: RolloutRuleComparator.SemVerGreaterOrEquals, label: '>= (SemVer)' },
            { value: RolloutRuleComparator.NumberEquals, label: '= (Number)' },
            { value: RolloutRuleComparator.NumberDoesNotEqual, label: '!= (Number)' },
            { value: RolloutRuleComparator.NumberLess, label: '< (Number)' },
            { value: RolloutRuleComparator.NumberLessOrEquals, label: '<= (Number)' },
            { value: RolloutRuleComparator.NumberGreater, label: '> (Number)' },
            { value: RolloutRuleComparator.NumberGreaterOrEquals, label: '>= (Number)' },
            { value: RolloutRuleComparator.SensitiveIsOneOf, label: 'IS ONE OF (Sensitive)' },
            { value: RolloutRuleComparator.SensitiveIsNotOneOf, label: 'IS NOT ONE OF (Sensitive)' },
        ];
    }
    onlyFirstTooltip() {
        return this.formGroup.root.get('allRolloutRulesCount').value > 20 || this.rolloutRules().controls.length > 10;
    }
    setting() {
        return this.formGroup.get('setting').value;
    }
    rolloutRules() {
        return this.formGroup.get('rolloutRules');
    }
    comparisonAttributes() {
        return this.formGroup.root.get('comparisonAttributes').value;
    }
    removeRule(index) {
        this.rolloutRules().removeAt(index);
        this.rolloutRules().markAsDirty();
    }
    onDrop(event) {
        const rolloutRules = this.rolloutRules().controls;
        moveItemInArray(rolloutRules, event.previousIndex, event.currentIndex);
        this.formGroup.markAsDirty();
    }
    attributeChanged(event, comparisonAttribute) {
        if (this.defaultComparisonAttributes.indexOf(event) !== -1) {
            if (comparisonAttribute.value !== event) {
                comparisonAttribute.setValue(event);
                this.formGroup.markAsDirty();
            }
        }
        else {
            if (comparisonAttribute.value !== '') {
                comparisonAttribute.setValue('');
                this.formGroup.markAsDirty();
            }
        }
    }
    getPlaceHolder(rolloutRule) {
        if (!rolloutRule.controls.comparisonAttribute.value) {
            return '';
        }
        switch (rolloutRule.controls.comparator.value) {
            case RolloutRuleComparator.IsOneOf:
            case RolloutRuleComparator.IsNotOneOf:
            case RolloutRuleComparator.SensitiveIsOneOf:
            case RolloutRuleComparator.SensitiveIsNotOneOf:
                switch (rolloutRule.controls.comparisonAttribute.value.toLowerCase()) {
                    case 'identifier':
                        return 'id1, id2, id3';
                    case 'email':
                        return 'mr.cat@configcat.com,mrs.cat@configcat.com';
                    case 'country':
                        return 'USA,UK';
                    default:
                        return 'value1, value2, value3';
                }
            case RolloutRuleComparator.Contains:
            case RolloutRuleComparator.DoesNotContain:
                switch (rolloutRule.controls.comparisonAttribute.value.toLowerCase()) {
                    case 'identifier':
                        return 'part-of-id';
                    case 'email':
                        return '@configcat.com';
                    case 'country':
                        return 'United';
                    default:
                        return 'part-of-value';
                }
            case RolloutRuleComparator.SemVerIsOneOf:
            case RolloutRuleComparator.SemVerIsNotOneOf:
                return '1.0.0, 2.0.1';
            case RolloutRuleComparator.SemVerLessOrEquals:
            case RolloutRuleComparator.SemVerLess:
            case RolloutRuleComparator.SemVerGreater:
            case RolloutRuleComparator.SemVerGreaterOrEquals:
                return '1.0.0';
            case RolloutRuleComparator.NumberEquals:
            case RolloutRuleComparator.NumberDoesNotEqual:
            case RolloutRuleComparator.NumberLess:
            case RolloutRuleComparator.NumberLessOrEquals:
            case RolloutRuleComparator.NumberGreater:
            case RolloutRuleComparator.NumberGreaterOrEquals:
                return '0';
        }
        return '';
    }
    getComparisonValueType(rolloutRule) {
        switch (rolloutRule.controls.comparator.value) {
            case RolloutRuleComparator.NumberEquals:
            case RolloutRuleComparator.NumberDoesNotEqual:
            case RolloutRuleComparator.NumberLess:
            case RolloutRuleComparator.NumberLessOrEquals:
            case RolloutRuleComparator.NumberGreater:
            case RolloutRuleComparator.NumberGreaterOrEquals:
                return 'number';
            default:
                return 'text';
        }
    }
    comparatorChanged(rolloutRule) {
        let validators = [];
        switch (rolloutRule.controls.comparator.value) {
            case RolloutRuleComparator.IsOneOf:
            case RolloutRuleComparator.IsNotOneOf:
            case RolloutRuleComparator.SensitiveIsOneOf:
            case RolloutRuleComparator.SensitiveIsNotOneOf:
            case RolloutRuleComparator.Contains:
            case RolloutRuleComparator.DoesNotContain:
                validators = [Validators.required, Validators.maxLength(65535)];
                break;
            case RolloutRuleComparator.SemVerIsOneOf:
            case RolloutRuleComparator.SemVerIsNotOneOf:
                validators = [Validators.required, this.validateSemVerList];
                break;
            case RolloutRuleComparator.SemVerLessOrEquals:
            case RolloutRuleComparator.SemVerLess:
            case RolloutRuleComparator.SemVerGreater:
            case RolloutRuleComparator.SemVerGreaterOrEquals:
                validators = [Validators.required, this.validateSemVer];
                break;
            case RolloutRuleComparator.NumberEquals:
            case RolloutRuleComparator.NumberDoesNotEqual:
            case RolloutRuleComparator.NumberLess:
            case RolloutRuleComparator.NumberLessOrEquals:
            case RolloutRuleComparator.NumberGreater:
            case RolloutRuleComparator.NumberGreaterOrEquals:
                validators = [Validators.required, Validators.min(-1.79769313486232E+308), Validators.max(1.79769313486232E+308)];
                break;
        }
        rolloutRule.controls.comparisonValue.setValidators(validators);
        rolloutRule.controls.comparisonValue.updateValueAndValidity();
    }
    isListType(rolloutRule) {
        switch (rolloutRule.controls.comparator.value) {
            case RolloutRuleComparator.IsOneOf:
            case RolloutRuleComparator.IsNotOneOf:
            case RolloutRuleComparator.SensitiveIsOneOf:
            case RolloutRuleComparator.SensitiveIsNotOneOf:
            case RolloutRuleComparator.SemVerIsOneOf:
            case RolloutRuleComparator.SemVerIsNotOneOf:
                return true;
            default:
                return false;
        }
    }
    validateSemVer(c) {
        if (valid(c.value) === null) {
            return {
                validateSemVer: {
                    valid: false
                }
            };
        }
        return null;
    }
    validateSemVerList(c) {
        if (!c.value) {
            return {
                validateSemVerList: {
                    valid: false
                }
            };
        }
        const semvers = c.value.split(',').map(s => s.trim());
        if (semvers.find(s => s && s !== '' && valid(s) === null)) {
            return {
                validateSemVerList: {
                    valid: false
                }
            };
        }
        return null;
    }
    popOut(rolloutRule) {
        this.dialog.open(ExpandDialogComponent, {
            width: '640px',
            data: {
                control: rolloutRule.controls.comparisonValue,
                multiline: this.isListType(rolloutRule)
            }
        });
    }
    trackByIndex(index) {
        return index;
    }
};
RolloutRulesComponent.ctorParameters = () => [
    { type: MatDialog }
];
__decorate([
    Input()
], RolloutRulesComponent.prototype, "formGroup", void 0);
__decorate([
    Input()
], RolloutRulesComponent.prototype, "readOnly", void 0);
__decorate([
    Input()
], RolloutRulesComponent.prototype, "maxRolloutRuleCount", void 0);
__decorate([
    Input()
], RolloutRulesComponent.prototype, "showVariationId", void 0);
RolloutRulesComponent = __decorate([
    Component({
        selector: 'configcat-rollout-rules',
        template: "<div [formGroup]=\"formGroup\">\r\n    <div cdkDropList (cdkDropListDropped)=\"onDrop($event)\" [cdkDropListDisabled]=\"readOnly\"\r\n        [cdkDropListData]=\"rolloutRules().controls\">\r\n        <div *ngFor=\"let rolloutRule of rolloutRules().controls; trackBy: trackByIndex; let i = index\" cdkDrag>\r\n            <div class=\"targeting-row\">\r\n                <div class=\"fields flex\">\r\n                    <div class=\"drag-handle\" cdkDragHandle data-tooltip=\"Drag here to reorder\" *ngIf=\"!readOnly\">\r\n                        <mat-icon>drag_indicator</mat-icon>\r\n                    </div>\r\n                    <div class=\"if\">\r\n                        <mat-chip-list [class.spacing]=\"i === 0\">\r\n                            <mat-chip class=\"chip\">{{ i === 0 ? 'IF' : 'ELSE IF' }}</mat-chip>\r\n                        </mat-chip-list>\r\n                        <a href=\"https://configcat.com/docs/advanced/user-object\" target=\"_blank\">user's</a>\r\n                    </div>\r\n\r\n                    <select class=\"custom-select attribute-select\" matTooltip=\"Comparison attribute\"\r\n                        matTooltipPosition=\"above\" [matTooltipDisabled]=\"onlyFirstTooltip() && i !== 0\"\r\n                        (change)=\"attributeChanged($event.target.value, rolloutRule.controls.comparisonAttribute)\">\r\n                        <option *ngFor=\"let comparisonAttribute of defaultComparisonAttributes\"\r\n                            [selected]=\"comparisonAttribute === rolloutRule.controls.comparisonAttribute.value\">\r\n                            {{comparisonAttribute}}</option>\r\n                        <option\r\n                            [selected]=\"defaultComparisonAttributes.indexOf(rolloutRule.controls.comparisonAttribute.value) === -1\">\r\n                            Custom\r\n                        </option>\r\n                    </select>\r\n\r\n\r\n                    <span class=\"attribute-span\" data-tooltip=\"Custom user attribute\"\r\n                        [hidden]=\"defaultComparisonAttributes.indexOf(rolloutRule.controls.comparisonAttribute.value) !== -1\">\r\n                        <span class=\"attribute-error\" *ngIf=\"rolloutRule.controls.comparisonAttribute.invalid\">*</span>\r\n                        <input [ngClass]=\"{'invalid': rolloutRule.controls.comparisonAttribute.invalid}\" type=\"text\"\r\n                            matInput [matAutocomplete]=\"auto\" [formControl]=\"rolloutRule.controls.comparisonAttribute\"\r\n                            class=\"attribute\" placeholder=\"e.g. Eye color\">\r\n                    </span>\r\n\r\n                    <mat-autocomplete #auto=\"matAutocomplete\">\r\n                        <mat-option *ngFor=\"let option of comparisonAttributes()\" [value]=\"option\">\r\n                            {{option}}\r\n                        </mat-option>\r\n                    </mat-autocomplete>\r\n\r\n                    <div class=\"select-wrapper\">\r\n                        <select class=\"custom-select\" [formControl]=\"rolloutRule.controls.comparator\"\r\n                            matTooltip=\"Comparator\" matTooltipPosition=\"above\"\r\n                            [matTooltipDisabled]=\"onlyFirstTooltip() && i !== 0\"\r\n                            (change)=\"comparatorChanged(rolloutRule)\">\r\n                            <optgroup label=\"TEXT\">\r\n                                <option *ngFor=\"let comparator of rolloutRuleComparators.slice(0, 4)\"\r\n                                    [ngValue]=\"comparator.value\">\r\n                                    {{comparator.label}}\r\n                                </option>\r\n                            </optgroup>\r\n                            <optgroup label=\"SENSITIVE TEXT\">\r\n                                <option *ngFor=\"let comparator of rolloutRuleComparators.slice(16, 18)\"\r\n                                    [ngValue]=\"comparator.value\">\r\n                                    {{comparator.label}}\r\n                                </option>\r\n                            </optgroup>\r\n                            <optgroup label=\"SEMANTIC VERSION\">\r\n                                <option *ngFor=\"let comparator of rolloutRuleComparators.slice(4, 10)\"\r\n                                    [ngValue]=\"comparator.value\">\r\n                                    {{comparator.label}}\r\n                                </option>\r\n                            </optgroup>\r\n                            <optgroup label=\"NUMBER\">\r\n                                <option *ngFor=\"let comparator of rolloutRuleComparators.slice(10, 16)\"\r\n                                    [ngValue]=\"comparator.value\">\r\n                                    {{comparator.label}}\r\n                                </option>\r\n                            </optgroup>\r\n                        </select>\r\n                        <div class=\"help\" matTooltip=\"Learn more about comparators\"\r\n                            [matTooltipDisabled]=\"onlyFirstTooltip() && i !== 0\">\r\n                            <a href=\"https://configcat.com/docs/advanced/targeting/#comparator\" target=\"_blank\">\r\n                                <mat-icon>help</mat-icon>\r\n                            </a>\r\n                        </div>\r\n                    </div>\r\n\r\n                    <div class=\"comparison-value\" data-tooltip=\"Comparison value\">\r\n                        <span class=\"error\" *ngIf=\"rolloutRule.controls.comparisonValue.invalid\">*</span>\r\n\r\n                        <input [ngClass]=\"{'invalid': rolloutRule.controls.comparisonValue.invalid}\"\r\n                            [formControl]=\"rolloutRule.controls.comparisonValue\"\r\n                            [type]=\"getComparisonValueType(rolloutRule)\" [placeholder]=\"getPlaceHolder(rolloutRule)\">\r\n                        <div class=\"expand\" matTooltip=\"Pop-out Editor\" (click)=\"popOut(rolloutRule)\" *ngIf=\"getComparisonValueType(rolloutRule)==='text'\">\r\n                            <mat-icon>height</mat-icon>\r\n                        </div>\r\n                    </div>\r\n\r\n                    <div class=\"delete\" matTooltip=\"Remove targeting rule\"\r\n                        [matTooltipDisabled]=\"onlyFirstTooltip() && i !== 0\" *ngIf=\"!readOnly\" (click)=\"removeRule(i)\">\r\n                        <mat-icon>cancel</mat-icon>\r\n                    </div>\r\n                </div>\r\n                <div class=\"value flex\">\r\n                    <div class=\"serve\" *ngIf=\"setting().settingType !== SettingType.Boolean\">\r\n                        <mat-chip-list>\r\n                            <mat-chip class=\"chip\">Serve:</mat-chip>\r\n                        </mat-chip-list>\r\n                    </div>\r\n\r\n                    <configcat-value class=\"served-value\" [valueFormControl]=\"rolloutRule.controls.value\"\r\n                        [setting]=\"setting()\" [showVariationId]=\"showVariationId\">\r\n                    </configcat-value>\r\n                </div>\r\n            </div>\r\n        </div>\r\n    </div>\r\n</div>",
        styles: [".cdk-drop-list-dragging .cdk-drag{transition:transform 250ms cubic-bezier(0,0,.2,1)}.cdk-drag-animating{transition:transform .3s cubic-bezier(0,0,.2,1)}.targeting-row{border:1px solid #d5d5d5;border-radius:4px;background-color:#fff;margin:.5em 1em;display:flex;position:relative;padding:.5em 0}.targeting-row .select-wrapper{position:relative}.targeting-row .help{position:absolute;cursor:pointer;top:-8px;left:166px}.targeting-row .help a{color:#a3c6f7}.targeting-row .help a:hover{color:#0068f0}.targeting-row .help mat-icon{font-size:18px}.targeting-row .chip{background-color:#67b1fc3d}.targeting-row .fields{justify-content:left;max-width:1060px;flex-grow:100}.targeting-row .fields .if{display:flex;align-items:center}.targeting-row .fields .if>*{margin:0 .5em}.targeting-row .fields .if .spacing{margin-right:46px}.targeting-row .fields .attribute-span{margin-left:.5em}.targeting-row .fields .attribute{width:170px;background-color:#fff;border:1px solid #e5e5e5;min-height:28px;min-width:170px;font-size:15px;padding:0 10px;font-family:Montserrat,Roboto,\"Helvetica Neue\",sans-serif;font-weight:300}.targeting-row .fields .invalid{border:1px solid #ff5252}.targeting-row .fields .attribute-error{color:#ff5252;margin-right:5px}.targeting-row .fields .delete{position:absolute;color:#d6a9a9;cursor:pointer;top:-8px;right:-12px;z-index:999}.targeting-row .fields .delete:hover{color:#ed5251}.targeting-row .fields .delete mat-icon{font-size:20px}.targeting-row .fields .drag-handle{position:absolute;cursor:-webkit-grab;cursor:-moz-grab;left:-20px;top:14px}.targeting-row .fields .drag-handle mat-icon{font-size:20px}.targeting-row .fields .custom-select{margin-left:.5em;width:170px;background-color:#fff;border:1px solid #e5e5e5;min-height:30px;min-width:170px;font-size:.8em;padding:0 5px;font-family:Montserrat,Roboto,\"Helvetica Neue\",sans-serif;font-weight:300}.targeting-row .fields .attribute-select{min-width:100px;width:100px;margin-left:0}.targeting-row .fields .comparison-value{flex-grow:100;min-width:200px;display:flex;align-items:center;margin-left:.5em}.targeting-row .fields .comparison-value .error{color:#ff5252;margin:0 5px}.targeting-row .fields .comparison-value input{background-color:#fff;border:1px solid #e5e5e5;min-height:28px;min-width:170px;flex-grow:100;font-size:15px;padding:0 10px;font-family:Montserrat,Roboto,\"Helvetica Neue\",sans-serif;font-weight:300}.targeting-row .fields .comparison-value .invalid{border:1px solid #ff5252}.targeting-row .value{margin-left:auto;flex-wrap:nowrap}.targeting-row .value .serve{margin:0 5px}.warn{padding-left:3em;font-size:.8em}.flex{display:flex;flex-wrap:wrap;align-items:center}.expand{position:absolute;right:2px;cursor:pointer;margin-left:-28px;z-index:1;color:#b3b3b3;background-color:#fff}.expand mat-icon{transform:rotate(45deg)}@media (max-width:1200px){.targeting-row{flex-wrap:wrap}.targeting-row .fields{max-width:500px}}@media (max-width:700px){.targeting-row{flex-direction:column;margin:.5em .3em}.targeting-row .help{left:161px}.targeting-row .fields{flex-direction:column;align-items:stretch}.targeting-row .fields .comparison-value,.targeting-row .fields .custom-select{margin:0}}"]
    })
], RolloutRulesComponent);

let ShowVariationidDialogComponent = class ShowVariationidDialogComponent {
    constructor(dialogRef, data) {
        this.dialogRef = dialogRef;
        this.data = data;
    }
    ngOnInit() {
    }
    close() {
        this.dialogRef.close({ button: 'cancel' });
    }
    copyToClipboard(inputElement) {
        inputElement.select();
        document.execCommand('copy');
        inputElement.setSelectionRange(0, 0);
    }
};
ShowVariationidDialogComponent.ctorParameters = () => [
    { type: MatDialogRef },
    { type: undefined, decorators: [{ type: Inject, args: [MAT_DIALOG_DATA,] }] }
];
ShowVariationidDialogComponent = __decorate([
    Component({
        selector: 'configcat-show-variationid-dialog',
        template: "<h3 mat-dialog-title>VariationID for A/B testing</h3>\r\n<div mat-dialog-content class=\"container\">\r\n  <mat-form-field appearance=\"outline\">\r\n    <input #variationIdInput matInput [value]=\"data.variationId\" readonly>\r\n    <button type=\"button\" matTooltip=\"Copy to clipboard\" (click)=\"copyToClipboard(variationIdInput)\" mat-icon-button\r\n            matSuffix>\r\n      <mat-icon>file_copy</mat-icon>\r\n    </button>\r\n  </mat-form-field>\r\n</div>\r\n<p>\r\n  This is a unique ID that belongs to <strong>{{data.settingShortKey}}</strong>'s\r\n  <strong>{{data.settingShortValue}}</strong> value.\r\n</p>\r\n<small>\r\n  Usage for A/B testing:\r\n  <ul>\r\n    <li>Your application should get these IDs from the ConfigCat SDK's <code>getVariationId()</code> and\r\n      <code>getAllVariationIds()</code> functions and pass them to your analytics tool.</li>\r\n    <li>Query the above ID in your analytics tool to identify the variation.</li>\r\n  </ul>\r\n</small>\r\n<div mat-dialog-actions>\r\n  <button type=\"button\" color=\"primary\" mat-raised-button (click)=\"close()\">Close</button>\r\n</div>\r\n",
        styles: [".container>*{width:100%}"]
    }),
    __param(1, Inject(MAT_DIALOG_DATA))
], ShowVariationidDialogComponent);

let ValueComponent = class ValueComponent {
    constructor(dialog) {
        this.dialog = dialog;
        this.sha1 = sha1_;
        this.showVariationId = false;
        this.SettingType = SettingType;
    }
    showVariationIdDialog() {
        let variationId = this.setting.key + '_';
        let value;
        let numberString;
        switch (this.setting.settingType) {
            case SettingType.Boolean:
                value = this.valueFormControl.value;
                variationId += ('' + value).toLowerCase();
                break;
            case SettingType.String:
                value = this.valueFormControl.value;
                variationId += value;
                break;
            case SettingType.Int:
                value = this.valueFormControl.value;
                numberString = '' + Math.trunc(value);
                variationId += numberString;
                break;
            case SettingType.Double:
                value = this.valueFormControl.value;
                numberString = '' + Math.trunc(value);
                const modulo = Math.abs(value % 1);
                if (modulo !== 0) {
                    numberString += ('.' + Math.trunc(modulo * 1000000));
                }
                variationId += numberString;
                break;
        }
        variationId = this.sha1(variationId).substring(0, 8);
        const shortKey = this.setting.key.length <= 20 ? this.setting.key : this.setting.key.substring(0, 17) + '...';
        const shortValue = value.toString().length <= 20 ? value.toString() : value.toString().substring(0, 17) + '...';
        const dialogRef = this.dialog.open(ShowVariationidDialogComponent, {
            width: '420px',
            data: { variationId, settingKey: this.setting.key, settingShortKey: shortKey, settingShortValue: shortValue }
        });
    }
    popOut() {
        this.dialog.open(ExpandDialogComponent, {
            width: '640px',
            data: { control: this.valueFormControl }
        });
    }
};
ValueComponent.ctorParameters = () => [
    { type: MatDialog }
];
__decorate([
    Input()
], ValueComponent.prototype, "valueFormControl", void 0);
__decorate([
    Input()
], ValueComponent.prototype, "setting", void 0);
__decorate([
    Input()
], ValueComponent.prototype, "showVariationId", void 0);
ValueComponent = __decorate([
    Component({
        selector: 'configcat-value',
        template: "<div class=\"flex\">\r\n  <div class=\"container\" [ngSwitch]=\"setting.settingType\">\r\n    <div *ngSwitchCase=\"SettingType.String\" class=\"with-validation\">\r\n      <span *ngIf=\"valueFormControl.invalid\" class=\"error\">*</span>\r\n      <input [ngClass]=\"{'invalid': valueFormControl.invalid}\" type=\"text\"\r\n             [formControl]=\"valueFormControl\" placeholder=\"my value\" name=\"value\">\r\n      <div class=\"expand\" (click)=\"popOut()\" matTooltip=\"Pop-out Editor\">\r\n        <mat-icon>height</mat-icon>\r\n      </div>\r\n    </div>\r\n    <div *ngSwitchCase=\"SettingType.Int\" class=\"with-validation\">\r\n      <span *ngIf=\"valueFormControl.invalid\" class=\"error\">*</span>\r\n      <input [ngClass]=\"{'invalid': valueFormControl.invalid}\" type=\"number\" [formControl]=\"valueFormControl\"\r\n             placeholder=\"42\" name=\"value\" />\r\n    </div>\r\n    <div *ngSwitchCase=\"SettingType.Double\" class=\"with-validation\">\r\n      <span *ngIf=\"valueFormControl.invalid\" class=\"error\">*</span>\r\n      <input [ngClass]=\"{'invalid': valueFormControl.invalid}\" type=\"number\" [formControl]=\"valueFormControl\"\r\n             placeholder=\"3.14\" name=\"value\">\r\n    </div>\r\n    <div *ngSwitchCase=\"SettingType.Boolean\" class=\"toggle\">\r\n      <label class=\"switch\">\r\n        <input type=\"checkbox\" [formControl]=\"valueFormControl\" name=\"value\" />\r\n        <span class=\"slider round\"></span>\r\n      </label>\r\n    </div>\r\n  </div>\r\n\r\n  <label class=\"variationId\" *ngIf=\"showVariationId\" matTooltip=\"Show Variation ID for A/B testing\"\r\n         (click)=\"showVariationIdDialog()\">\r\n    <mat-icon>insert_chart_outlined</mat-icon>\r\n  </label>\r\n  <div *ngIf=\"setting.settingType === SettingType.Boolean && !showVariationId\" class=\"spacer\"></div>\r\n\r\n</div>\r\n",
        styles: [".switch{position:relative;width:66px;height:30px;margin-left:4px;margin-top:0;margin-bottom:0}.switch input{display:none}.slider{position:absolute;cursor:pointer;top:0;left:0;right:0;bottom:0;background-color:#ff5252;transition:.4s;width:66px;height:30px}.slider:before{position:absolute;content:\"\";height:22px;width:22px;left:4px;bottom:4px;background-color:#fff;transition:.4s}input:checked+.slider{background-color:#9acd32}input:focus+.slider{box-shadow:0 0 1px #9acd32}input:checked+.slider:before{transform:translateX(36px)}.slider.round{border-radius:34px}.slider.round:before{border-radius:50%}.slider:after{content:\"Off\";color:#fff;display:block;position:absolute;transform:translate(-14%,-50%);top:50%;left:50%;font-size:10px;font-family:Verdana,sans-serif}input:checked+.slider:after{content:\"On\";transform:translate(-91%,-50%)}", ".flex{display:flex;margin-right:20px;align-items:center}.flex .spacer{width:15px}.flex .container .with-validation{display:flex;flex-wrap:nowrap}.flex .container .toggle{width:66px;height:30px}.flex .container input{background-color:#fff;border:1px solid #e5e5e5;min-height:28px;font-size:15px;padding:0 10px;font-family:Montserrat,Roboto,\"Helvetica Neue\",sans-serif;font-weight:300;min-width:170px}.flex .container .invalid{border:1px solid #ff5252}.flex .container .error{color:#ff5252;margin-right:5px}@media (max-width:1200px){.flex .container input{min-width:auto}}.flex .variationId{margin-left:5px;cursor:pointer;color:#888;display:flex;align-items:center}.flex .expand{position:relative;top:2px;height:24px;cursor:pointer;margin-left:-28px;z-index:1;color:#b3b3b3;background-color:#fff}.flex .expand mat-icon{transform:rotate(45deg)}"]
    })
], ValueComponent);

let ConfigCatSettingValuesUiModule = class ConfigCatSettingValuesUiModule {
};
ConfigCatSettingValuesUiModule = __decorate([
    NgModule({
        declarations: [
            SettingValuesComponent,
            SettingValueComponent,
            RolloutPercentageItemsComponent,
            RolloutRulesComponent,
            ValueComponent,
            ReasonDialogComponent,
            ShowVariationidDialogComponent,
            ConcurrentDialogComponent,
            SaveSnackbarComponent,
            ExpandDialogComponent
        ],
        imports: [
            CommonModule,
            FormsModule,
            ReactiveFormsModule,
            BrowserAnimationsModule,
            MatIconModule,
            MatTooltipModule,
            MatChipsModule,
            MatButtonModule,
            MatAutocompleteModule,
            MatFormFieldModule,
            MatInputModule,
            DragDropModule,
            RouterModule,
            MatSnackBarModule,
            MatDialogModule
        ],
        exports: [
            SettingValuesComponent
        ]
    })
], ConfigCatSettingValuesUiModule);

/*
 * Public API Surface of configcat-settingvalues-ui
 */

/**
 * Generated bundle index. Do not edit.
 */

export { ConfigCatSettingValuesUiModule, IntegrationLinkType, RolloutRuleComparator, SettingType, SettingValuesComponent as ɵa, FormHelper as ɵb, SettingValueComponent as ɵc, RolloutPercentageItemsComponent as ɵd, RolloutRulesComponent as ɵe, ValueComponent as ɵf, ReasonDialogComponent as ɵg, ShowVariationidDialogComponent as ɵh, ConcurrentDialogComponent as ɵi, SaveSnackbarComponent as ɵj, ExpandDialogComponent as ɵk };
//# sourceMappingURL=ng-configcat-settingvalues-ui.js.map
